#ifndef SPRITEHEADER
#define SPRITEHEADER
#include <windows.h>
#include <gl/gl.h>

#include <map>
#include <string>
#include <vector>
using namespace std;


/*
To use sprites:

first make your frame and anim sets like this:
	FrameSet fs;
	AnimationSet as(fs);

sprites can then be instansiated by associating them with an animset

	Sprite s(as);
	Sprite s2(as);

To load data from an xml file, do this:

	fs.LoadFromXML("frameset1.xml");
	as.LoadFromXML("animset1.xml");

make sure to load the frameset before its animset(s)!!!


In the game loop, you have to update and draw sprites:

to update (dt is a double in delta seconds):
	s.IncrementTimer(dt);

finally to draw:
	s.Draw();


Of course, you'll control sprites using the Set* functions, and the playanim

Currently, a sprite will just stop playing after its animation finishes. We can add some queueing if we need it
*/



class Frame
{
public:
	friend class FrameSet;
	Frame() {};
	~Frame() {};
	void Draw(int x, int y,float rot, float sx, float sy,float a,float r, float g, float b,bool flipLR,bool flipUD) const;
	void DrawPercent(int x, int y, float rot, float sx, float sy,float a,float r, float g, float b, float percent) const;
private:

	int offset[2];
	unsigned int size[2];
	
	//texture coords
	float tcUL[2];
	float tcLR[2];

	GLuint texID;
	bool alpha;
	string name;
};

class FrameSet
{
public:
	FrameSet() {};
	~FrameSet();
	bool LoadFromXML(const string & fname);
	const Frame * GetFrame(const string & name) const;

private:
	map<string, Frame*> frames;
	vector<GLuint> ids;
};

class Animation
{

public:
	friend class AnimationSet;
	Animation() :length(0) {};
	~Animation() {};
	double GetLength() const {return length;};
	//string GetFrameName(double time) const;
	const Frame * GetFrame(double time) const;

private:
	vector<double> times;
	vector<string> frameNames;
	vector<const Frame *> frames;
	double length;
};

class AnimationSet
{
public:	
	AnimationSet(FrameSet & fs) : frameset(fs) {};
	~AnimationSet();

	bool GetFrame(const string & name, const Frame * & frame) const;
	bool GetFrame(const string & name, double time, const Frame * & frame) const;
	bool LoadFromXML(const string & fname);
	const Animation * GetAnimation(const string & name) const;

private:
	FrameSet & frameset;
	map<string, Animation*> anims;	
};


class Sprite
{
public:
	Sprite(AnimationSet & as) :
		flipLR(false), flipUD(false), rot(0), alpha(1),
		currentTime(0), startTime(0), lastFrame(0),
		currentAnim(0), nextAnim(0), animset(as),
		stale(false), playing(false)
	{
		scale[0] = 1;
		scale[1] = 1;

		pos[0] = 0;
		pos[1] = 0;

		colour[0] = 1;
		colour[1] = 1;
		colour[2] = 1;
	}

	~Sprite() {};
	
	void IncrementTimer(double dt);
	bool SetFrame(const string & fname);
	bool PlayAnimation(const string & fname);
	void SetPos(float x, float y);
	void SetRot(float r);
	void SetScale(float sx, float sy);
	void SetAlpha(float a);
	void Draw();
	bool IsPlaying() {return playing;};
	void SetColour(float r, float g, float b);
	void FlipUD(bool val) {flipUD = val;};
	void FlipLR(bool val) {flipLR = val;};
	void SetNextAnimation(const string & name);
	void ClearNextAnimation();
	string GetAnimName() {return animName;};
private:
	bool flipLR;
	bool flipUD;
	float scale[2];
	float rot;
	float pos[2];
	float alpha;
	float colour[3];

	double currentTime;
	double startTime;
	const Frame * lastFrame;
	const Animation * currentAnim;
	const Animation * nextAnim;
	AnimationSet & animset;
	bool stale;
	bool playing;
	string animName,nextAnimName;
};



#endif